!#################################################################
!
! this budget routine defines the global budgets
! for instance, the budget in certain lattitude bands (SH, NH) 
! and heights (BL, FT, STRAT)
! Apart from that, the budget in the finest zoom region can be defined
! There is a difficulty with the 'INTERFACE' regions. Here, the
! emissions, conv, chemistry are performed in the parent resolutions.
! A straightforward separation between advection and these
! processes becomes therefore problamatic. Therefore, the advection
! budget is not calculated for now.
!
! The budget is defined according to the 'coarsest' resolution (e.g. 6x4)
! The zoom regions will be defined according to the defined regions.
! 
!### macro's #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module budget_global

  use dims
  use chem_param, only : names, ra, nmark, marknam, njnum, jnam, nreac
  use chem_param, only : ratnam, nreacw, rwnam, ntrace, ntracet

  implicit none

  ! --- interface ---

  private

  ! public routines
  public :: ini_zoneg, diagbudg, budget_transportg

  ! public types
  public :: buddep_data, budg_data, budflux_data

  ! public variables
  public :: init_mass, buddep_dat, budg_dat, nzon_vg, nbudg, nbud_vg
  public :: sum_deposition, sum_wet, sum_chemistry,sum_emission
  public :: sum_update, sum_advection, sum_stratosphere
  public :: iflux1, iflux2, jflux1, jflux2, lflux1, lflux2
  public :: budemig, budstratg, budrjg, budrrg, budrwg, budmarkg
  public :: budget_flux, apply_budget_global

  ! --- module variables ---

  logical               :: apply_budget_global = .false.
  integer,parameter     :: nbudg=7, nbud_vg=4

  !  deposition data on gridbasis
  type buddep_data
     real,dimension(:,:,:,:),pointer    :: lsp     ! im,jm,nbud_vg,ntrace
     real,dimension(:,:,:,:),pointer    :: cp      ! im,jm,nbud_vg,ntrace
     real,dimension(:,:,:,:),pointer    :: cond    ! im,jm,nbud_vg,ntrace
     real,dimension(:,:,:),pointer      :: dry     ! im,jm,ntrace
  end type buddep_data

  type(buddep_data),dimension(nregions),target  :: buddep_dat, buddep_dat_all

  real,dimension(nbudg,nbud_vg,ntracet)        :: rmoldg,rmnewg

  ! budget for tracer 1 for output checking.......
  real,dimension(nregions)  :: init_mass = 0.0    ! only for tracer 1, in kg
  real,dimension(nregions)  :: sum_deposition = 0.0    
  real,dimension(nregions)  :: sum_wet = 0.0    
  real,dimension(nregions)  :: sum_chemistry = 0.0
  real,dimension(nregions)  :: sum_update = 0.0
  real,dimension(nregions)  :: sum_advection = 0.0
  real,dimension(nregions)  :: sum_stratosphere = 0.0
  real,dimension(nregions)  :: sum_emission = 0.0

  !    budrrg  : accumulator for chemical reaction fluxes
  !    budrjg  : accumulator for photolysis reactions
  !    buddryg : accumulator for dry deposition losses
  !    budwetg : accumulator for wet deposition losses
  !    budemig : accumulator for emissions entering calculations
  !    budstratg : accumulator for stratospheric losses and gains
  !    budconcg  : average concentrations
  !    budchngg  : change of concentration
  !    budinig   : initial concentration at begin of run
  !    budconvg : convection in and out of region per tracer
  !
  real, dimension(nbudg,nbud_vg,nreac)  :: budrrg
  real, dimension(nbudg,nbud_vg,nreac)  :: budrrg_all
  real, dimension(nbudg,nbud_vg,njnum)  :: budrjg
  real, dimension(nbudg,nbud_vg,njnum)  :: budrjg_all
  real, dimension(nbudg,nbud_vg,ntrace) :: buddryg
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_lsp
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_cp
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_nat
  real, dimension(nbudg,nbud_vg,ntrace) :: budemig
  real, dimension(nbudg,nbud_vg,ntrace) :: budstratg
  real, dimension(nbudg,nbud_vg,ntrace) :: budconcg
  real, dimension(nbudg,nbud_vg,ntrace) :: budinig
  real, dimension(nbudg,nbud_vg,ntrace) :: buddryg_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_lsp_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_cp_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budwetg_nat_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budemig_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budstratg_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budconcg_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budinig_all
  real, dimension(nbudg,nbud_vg,ntrace) :: budchngg = 0.0
  real, dimension(nbudg,nbud_vg,ntrace) :: budchngg_all = 0.0
  real, dimension(nbudg,nbud_vg,nmark)  :: budmarkg
  real, dimension(nbudg,nbud_vg,nmark)  :: budmarkg_all
  real, dimension(nbudg,nbud_vg,nreacw) :: budrwg
  real, dimension(nbudg,nbud_vg,nreacw) :: budrwg_all
  real, dimension(nbudg,nbud_vg,ntracet):: budadvxg
  real, dimension(nbudg,nbud_vg,ntracet):: budadvyg
  real, dimension(nbudg,nbud_vg,ntracet):: budadvzg
  real, dimension(nbudg,nbud_vg,ntracet):: budadvxg_all
  real, dimension(nbudg,nbud_vg,ntracet):: budadvyg_all
  real, dimension(nbudg,nbud_vg,ntracet):: budadvzg_all
  real, dimension(nbudg,nbud_vg,ntracet):: budconvg
  real, dimension(nbudg,nbud_vg,ntracet):: budconvg_all

  character(len=20), dimension(nbudg)   :: zone_nameg
  character(len=20), dimension(nbud_vg) :: zone_namvg

  type budg_data
     ! defines the horizontal region definition
     integer,dimension(:,:),pointer      :: nzong 
  end type budg_data

  type(budg_data),dimension(nregions),target    :: budg_dat

  integer,dimension(lm(1))          :: nzon_vg
  integer                           :: lglob !WP! for offset on level domain

  ! added dec 2004: fluxes of all species through some predefined levels:
  type budflux_data
     real,dimension(:,:,:),pointer      :: flux_x1   ! jm,lm,ntracet
     real,dimension(:,:,:),pointer      :: flux_x2   ! jm,lm,ntracet
     real,dimension(:,:,:),pointer      :: flux_y1   ! im,lm,ntracet
     real,dimension(:,:,:),pointer      :: flux_y2   ! im,lm,ntracet
     real,dimension(:,:,:),pointer      :: flux_z1   ! im,jm,ntracet
     real,dimension(:,:,:),pointer      :: flux_z2   ! im,jm,ntracet
  end type budflux_data

  type(budflux_data),dimension(nregions),target  :: budget_flux, budget_flux_all

  integer, dimension(nregions)          :: iflux1, iflux2  ! calculate flux flowing in from west (2x)
  integer, dimension(nregions)          :: jflux1, jflux2  ! calculate flux flowing in from south (2x)
  integer, dimension(nregions)          :: lflux1, lflux2  ! calculate flux flowing in from below (2x)

contains



  subroutine ini_zoneg
    !
    ! user defined structure of budget zones
    !
    use global_data, only : outdir, rcfile
    use io_hdf,   only : io_write, DFACC_CREATE, DFNT_INT32
    use toolbox,  only : lvlpress, escape_tm
    use GO, only : TrcFile, Init, Done, ReadRc
#ifdef MPI
    use mpi_const,only : root,myid
#endif

    implicit none

    ! local
    integer :: i,j,n, nix,niy,region
    integer :: io,sfstart,sfend,istat,sfsnatt
    integer :: ilvl1, ilvl2, lats, ilvl3
    real    :: psurf, plev, rlat, rlon
    integer :: dyr, dxr

    type(TrcFile)       ::  rcF
    integer             ::  status

    ! start
    call Init( rcF, rcfile, status )
    if (status/=0) stop 'STOP in ini_zoneg'

    call ReadRc(rcF, 'apply.budget.global', apply_budget_global, status )
    if (status/=0) stop 'STOP in ini_zoneg'

    call Done( rcF, status )
    if (status/=0) stop 'STOP in ini_zoneg'

    if (.not.apply_budget_global) return

    region = 1
    !
    !
    zone_namvg(1)='BL'
    zone_namvg(2)='FT'                         
    zone_namvg(3)='STRAT'
    !
    zone_nameg(1)='SH 90-30'
    zone_nameg(2)='30S-30N '
    zone_nameg(3)='NH 90-30'
    zone_nameg(4)='Europe'
    zone_nameg(5)='N-america'
    zone_nameg(6)='Asia'
    zone_nameg(7)='Africa'

    psurf = 98400.0
    plev  = 85000.0
    ilvl1 = lvlpress(region,plev, psurf)
    plev  = 50000.0
    ilvl2 = lvlpress(region,plev, psurf)
    plev  = 10000.0
    ilvl3 = lvlpress(region,plev, psurf)
    nzon_vg(1:ilvl1) = 1 !boundary layer 
    nzon_vg(ilvl1+1:ilvl2)= 2 !850-->500 hPa
    nzon_vg(ilvl2+1:ilvl3)= 3 !500-->100 hPa
    nzon_vg(ilvl3+1:lm(1))= 4 !100 hPa-->TOA
    lflux1(:) = ilvl1+1
    lflux2(:) = ilvl2+1



    ! allocate arrays for horizontal decomposition of the regions:

    do n=1,nregions
       allocate( budg_dat(n)%nzong(im(n),jm(n)))
       allocate( buddep_dat(n)%lsp(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat(n)% cp(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat(n)% cond(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat(n)%dry(im(n),jm(n),ntrace))
       allocate( buddep_dat_all(n)%lsp(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat_all(n)% cp(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat_all(n)% cond(im(n),jm(n),nbud_vg,ntrace))
       allocate( buddep_dat_all(n)%dry(im(n),jm(n),ntrace))
       ! allocate flux data:
       allocate( budget_flux(n)%flux_x1(jm(n),lm(n),ntracet))
       allocate( budget_flux(n)%flux_y1(im(n),lm(n),ntracet))
       allocate( budget_flux(n)%flux_z1(im(n),jm(n),ntracet))
       allocate( budget_flux(n)%flux_x2(jm(n),lm(n),ntracet))
       allocate( budget_flux(n)%flux_y2(im(n),lm(n),ntracet))
       allocate( budget_flux(n)%flux_z2(im(n),jm(n),ntracet))
       allocate( budget_flux_all(n)%flux_x1(jm(n),lm(n),ntracet))
       allocate( budget_flux_all(n)%flux_y1(im(n),lm(n),ntracet))
       allocate( budget_flux_all(n)%flux_z1(im(n),jm(n),ntracet))
       allocate( budget_flux_all(n)%flux_x2(jm(n),lm(n),ntracet))
       allocate( budget_flux_all(n)%flux_y2(im(n),lm(n),ntracet))
       allocate( budget_flux_all(n)%flux_z2(im(n),jm(n),ntracet))
       budget_flux(n)%flux_x1(:,:,:) = 0.0
       budget_flux(n)%flux_y1(:,:,:) = 0.0
       budget_flux(n)%flux_z1(:,:,:) = 0.0
       budget_flux(n)%flux_x2(:,:,:) = 0.0
       budget_flux(n)%flux_y2(:,:,:) = 0.0
       budget_flux(n)%flux_z2(:,:,:) = 0.0
    end do

    ! copy zone information to all regions...

    do n = 1,nregions
       dyr = nint(dy/yref(n))
       dxr = nint(dx/xref(n))
       do j=1,jm(n)
          lats = ybeg(n) + (j-1)*dyr
          select case(lats)
          case(-90:-31)
             budg_dat(n)%nzong(:,j) = 3  !SH 90-30
          case(-30:29)
             budg_dat(n)%nzong(:,j) = 2  !-30->30
          case(30:90)
             budg_dat(n)%nzong(:,j) = 1  !NH 30-90
          case default
             call escape_tm('ini_zoneg: Wrong latitude in budget routine')
          end select
       end do
       do j=1,jm(n)
          rlat = ybeg(n) + (float(j)-0.5)*dyr
          if(  nint(ybeg(n) + float(j-1)*dyr) == 34) jflux1(n) = j
          if(  nint(ybeg(n) + float(j-1)*dyr) == 62) jflux2(n) = j
          !if(  nint(ybeg(n) + float(j)*dyr) == 62)   jflux2(n) = j !CMK corrected
          do i=1,im(n)
             rlon = xbeg(n) + (float(i)-0.5)*dxr
             if(  nint(xbeg(n) + float(i-1)*dxr) == -12) iflux1(n) = i
             if(  nint(xbeg(n) + float(i-1)*dxr) == 36)  iflux2(n) = i   ! corrected CMK
             !cmk if(  nint(xbeg(n) + float(i)*dxr) == 36)  iflux2(n) = i   ! corrected CMK
             if(rlon > -12. .and. rlon < 36. .and. rlat > 34. .and. rlat < 62.)  then
                budg_dat(n)%nzong(i,j) = 4   ! EUROPE
             elseif(rlon > -126. .and. rlon < -66. .and. rlat > 22. .and. rlat < 62.)  then
                budg_dat(n)%nzong(i,j) = 5   ! NAM
             elseif(rlon > 72. .and. rlon < 150. .and. rlat > -10. .and. rlat < 50.)  then
                budg_dat(n)%nzong(i,j) = 6   ! Asia
             elseif(rlon > -18. .and. rlon < 48. .and. rlat > -34. .and. rlat < 34.)  then
                budg_dat(n)%nzong(i,j) = 7   ! Africa
             endif
          enddo
       enddo
        
#ifdef MPI   
       if ( myid == root ) then
#endif
          io=SFSTART(trim(outdir)//'/regionsg_'//region_name(n)//'.hdf', DFACC_CREATE)
          istat = sfsnatt(io,'im',    DFNT_INT32, 1, im(n))
          istat = sfsnatt(io,'jm',    DFNT_INT32, 1, jm(n))
          istat = sfsnatt(io,'lm',    DFNT_INT32, 1, lm(n))
          call io_write(io,im(n),'im',jm(n),'jm', &
               budg_dat(n)%nzong,'region_global')
          print *,'ini_zoneg: closing output file',SFEND(io)
#ifdef MPI   
       end if
#endif
    end do

  end subroutine ini_zoneg



  subroutine diagbudg(region,k)
    !-----------------------------------------------------------------------
    !
    !
    !       interface
    !       ---------
    !       call call_diagbud(region,k)
    !       if k=1   reset budget variables budrr,budrj,buddry,
    !                budwet,budemi,budstrat,budconc,budchng, 
    !                initialise budini, allocate memory
    !       if k=2   output of budget variables, free memory
    !       if k=3   accumulate deposition budget
    !
    !       method
    !       ------
    !       output of time accumulated budget files for postprocessing
    !
    !       externals
    !       ---------
    !       none
    !
    !       reference
    !       ---------
    !       none
    !-----------------------------------------------------------------------
    use global_data, only : outdir
    use global_data, only : region_dat, mass_dat
    use io_hdf,      only : io_write
    use io_hdf,      only : DFACC_CREATE, DFNT_INT32, DFNT_INT64, DFNT_CHAR
    use toolbox,     only : escape_tm
#ifdef MPI
    use mpi_const
#endif

    implicit none

    ! in/out
    integer,intent(in) :: region,k

    ! local
    real,dimension(:,:,:,:),pointer    ::rm
    integer,dimension(:,:) ,pointer    ::zoomed,edge
    integer :: nzone,nzone_v
    integer :: i,j,l,n,nt,iy

    integer,parameter       :: nproces=15     ! number of budeget processes

    character(len=10), dimension(nproces),parameter :: proces_name=(/&
         'ratnam    ','jnam      ','rwnam     ','marknam   ','names     ', &
         'names     ','names     ','names     ','names     ','names     ', &
         'names     ','names     ','names     ','names     ','names     '/)
    integer,dimension(nproces),parameter :: proces_nr = (/ &
          nreac,   njnum,  nreacw,   nmark,  ntrace, &
         ntrace,  ntrace,  ntrace,  ntrace,  ntrace, &
         ntrace, ntracet, ntracet, ntracet, ntracet    /)
    integer   :: sfstart, sfend, sfsnatt, sfscatt, io, istat
    integer   :: communicator,offsetn,offsetl,lglob,nglob,root_id,lmr

    save

    ! start
    if (.not.apply_budget_global) return

    !WP diagbud can be called during chemistry, which is parallel 
    !   over levels. However, a version parallel over 
    !WP tracers is also implemented for future changes.

#ifdef MPI
    which_par=previous_par(region)

    if ( which_par=='tracer'.and.ntracetloc==0) return 
    if (which_par=='levels'.and.lmloc==0) return  !WP! 
#endif

#ifdef MPI

    if (which_par=='tracer') then

       rm => mass_dat(region)%rm_t
       lmr = lm(region)
       nt=ntracetloc
       communicator=com_trac  !WP! assign com_trac as communicator
       root_id=root_t
       offsetn=sum(ntracet_ar(0:myid-1) )  !offset for global value of n
       offsetl=0  ! no offset for levels
    else if (which_par=='levels') then

       rm => mass_dat(region)%rm_k
       lmr = lmloc
       nt=ntracet
       communicator=com_lev  !WP! assign com_lev as communicator
       root_id=root_k
       offsetl=sum(lmar(0:myid-1) )  ! offset for global value of l
       offsetn=0    ! no offset for tracers

    end if

#else
    rm => mass_dat(region)%rm_t
    lmr = lm(region)
    nt=ntracet
    offsetn=0  !
    offsetl=0  ! no offset for levels
#endif

    select case(k)

    case(1) !               _____________________case 1: initialise

#ifdef MPI 
       if ( which_par /= 'tracer' ) &
            call escape_tm( &
            'diagbudg: initializing budgets should be done on tracer domain') 
#endif

       if ( region == 1 ) then     !only for region = 1....
          !write(kmain,*) 'diagbudg: reset budget variables'
          budrrg=0.
          budrrg_all=0.
          budrjg=0.
          budrjg_all=0.
          budemig=0.
          budemig_all=0.
          budstratg=0.
          budstratg_all=0.
          budconcg=0.
          budconcg_all=0.
          budinig=0.
          budinig_all=0.
          budmarkg=0.
          budmarkg_all=0.
          budrwg=0.
          budrwg_all=0.
          budconvg=0.0
          budconvg_all=0.0
          budadvxg=0.0
          budadvxg_all=0.0
          budadvyg=0.0
          budadvyg_all=0.0
          budadvzg=0.0
          budadvzg_all=0.0
          !
          do n=1,nregions
             buddep_dat(n)%lsp = 0.0
             buddep_dat(n)%cp  = 0.0
             buddep_dat(n)%cond  = 0.0
             buddep_dat(n)%dry = 0.0
             buddep_dat_all(n)%lsp = 0.0
             buddep_dat_all(n)%cp  = 0.0
             buddep_dat_all(n)%cond  = 0.0
             buddep_dat_all(n)%dry = 0.0
          end do
       end if   !only for region 1   

       zoomed => region_dat(region)%zoomed
       edge   => region_dat(region)%edge

       ! tracer 1 budget...
#ifdef MPI 
       if ( myid == pe_first_tracer ) &
#endif
       init_mass(region) = sum(rm(1:im(region),1:jm(region),1:lmr,1))
       sum_emission(region)   = 0.0    ! only for tracer 1
       sum_deposition(region) = 0.0    ! 
       sum_wet(region)        = 0.0    ! 
       sum_chemistry(region)  = 0.0
       sum_update(region)     = 0.0
       sum_advection(region)  = 0.0
       !
       ! initialise budinig
       !
       do n=1,nt
          nglob=n+offsetn
          do l=1,lmr
             nzone_v=nzon_vg(l)
             do j=1,jm(region)
                do i=1,im(region)
                   !skip zoomed part of the region
                   if ( zoomed(i,j) /= region .or. edge(i,j) >= 1 ) cycle
                   ! include the edges....parent takes care!
                   nzone=budg_dat(region)%nzong(i,j)
                   budinig(nzone,nzone_v,nglob) = &
                        budinig(nzone,nzone_v,nglob) + &
                        rm(i,j,l,n)/ra(nglob)*1000.
                end do !i
             end do!j
          end do!l
       end do ! n 

       nullify(rm)
       nullify(zoomed)
       nullify(edge)

    case(2) !____________________________write and close, free memory
       !

       zoomed => region_dat(region)%zoomed
       edge   => region_dat(region)%edge

       if ( region == 1 ) budconcg = 0.0
       do n=1,nt
          nglob=n+offsetn !WP! offset is zero on levels domain
          do l=1,lmr
             lglob=l+offsetl !WP! offset is zero on tracer domain
             nzone_v=nzon_vg(lglob)
             do j=1,jm(region)
                do i=1,im(region)
                   if (zoomed(i,j)/=region .or. edge(i,j) >= 1) cycle  
                   nzone=budg_dat(region)%nzong(i,j)
                   budconcg(nzone,nzone_v,nglob) = &
                        budconcg(nzone,nzone_v,nglob) + &
                        rm(i,j,l,n)/ra(nglob)*1000.
                end do !i
             end do!j
          end do!l
       end do !n
       !

       nullify(rm)
       nullify(zoomed)
       nullify(edge)

       !calculate change and write....only for the last region
       if ( region == nregions ) then

          do  n=1,nt
             nglob=n+offsetn  !WP! offset is zero on levels domain
             do i=1,nbudg
                do iy=1,nbud_vg
                   budchngg(i,iy,nglob) = budconcg(i,iy,nglob) - &
                        budinig(i,iy,nglob)
                end do ! iy
             end do !i
          end do !n

#ifdef MPI   
          call gather_budget
#endif

#ifdef MPI   
          if (myid==root_id) then     !WP! only root writes
#endif

             !
             ! write to hdf file
             !
             io = sfstart(trim(outdir)//'/budget_global.hdf',DFACC_CREATE)
             istat = sfsnatt(io,'nregions' ,   DFNT_INT32, 1, nregions)
             istat = sfsnatt(io,'itau',     DFNT_INT64, 1, itau)
             istat = sfsnatt(io,'nbud',     DFNT_INT32, 1, nbudg)
             istat = sfsnatt(io,'nbud_v',   DFNT_INT32, 1, nbud_vg)
             istat = sfsnatt(io,'nreac',    DFNT_INT32, 1, nreac)
             istat = sfsnatt(io,'njnum',    DFNT_INT32, 1, njnum)
             istat = sfsnatt(io,'ntrace',   DFNT_INT32, 1, ntrace)
             istat = sfsnatt(io,'nmark',    DFNT_INT32, 1, nmark)
             istat = sfsnatt(io,'nreacw',   DFNT_INT32, 1, nreacw)
             istat = sfsnatt(io,'idate' ,   DFNT_INT32, 6, idate)
             istat = sfsnatt(io,'idatei' ,  DFNT_INT32, 6, idatei)
             istat = sfsnatt(io,'idatee' ,  DFNT_INT32, 6, idatee)
             istat = sfsnatt(io,'proces_nr' , DFNT_INT32, nproces, proces_nr)
             istat = sfscatt(io,'proces_name',DFNT_CHAR,nproces*10,proces_name)
             istat = sfscatt(io,'zone_name'  ,DFNT_CHAR, nbudg*20, zone_nameg)
             istat = sfscatt(io,'zone_namv'  ,DFNT_CHAR,nbud_vg*20, zone_namvg)
             istat = sfscatt(io,'names'      ,DFNT_CHAR, ntrace*8, names)
             istat = sfscatt(io,'ratnam'     ,DFNT_CHAR, nreac*8, ratnam)
             istat = sfscatt(io,'jnam'       ,DFNT_CHAR, njnum*8, jnam)
             istat = sfscatt(io,'marknam'    ,DFNT_CHAR, nmark*8, marknam)
             istat = sfscatt(io,'rwnam'      ,DFNT_CHAR, nreacw*8, rwnam)
             istat = sfsnatt(io,'iflux1'     ,DFNT_INT32, nregions, iflux1)
             istat = sfsnatt(io,'iflux2'     ,DFNT_INT32, nregions, iflux2)
             istat = sfsnatt(io,'jflux1'     ,DFNT_INT32, nregions, jflux1)
             istat = sfsnatt(io,'jflux2'     ,DFNT_INT32, nregions, jflux2)
             istat = sfsnatt(io,'lflux1'     ,DFNT_INT32, nregions, lflux1)
             istat = sfsnatt(io,'lflux2'     ,DFNT_INT32, nregions, lflux2)

#ifdef MPI
             !
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  njnum,'njnum',budrjg_all,'budrj')  
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nreac,'nreac',budrrg_all,'budrr')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nreacw,'nreacw',budrwg_all,'budrw')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nmark,'nmark',budmarkg_all,'budmark')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',buddryg_all,'buddry')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_lsp_all,'budwet_lsp')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_cp_all,'budwet_cp')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_nat_all,'budwet_nat')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budemig_all,'budemi')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budstratg_all,'budstrat')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budconcg_all,'budconc')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budchngg_all,'budchng')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budinig_all,'budini')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvxg_all,'budadvx')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvyg_all,'budadvy')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvzg_all,'budadvz')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budconvg_all,'budconv')

#else
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  njnum,'njnum',budrjg,'budrj')  
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nreac,'nreac',budrrg,'budrr')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nreacw,'nreacw',budrwg,'budrw')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  nmark,'nmark',budmarkg,'budmark')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',buddryg,'buddry')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_lsp,'budwet_lsp')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_cp,'budwet_cp')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budwetg_nat,'budwet_nat')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budemig,'budemi')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budstratg,'budstrat')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budconcg,'budconc')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budchngg,'budchng')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntrace,'ntrace',budinig,'budini')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvxg,'budadvx')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvyg,'budadvy')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budadvzg,'budadvz')
             call io_write(io,nbudg,'nbud',nbud_vg,'nbud_v', &
                  ntracet,'ntracet',budconvg,'budconv')
             !
#endif
             do n=1,nregions   !write and deallocate fields
#ifdef MPI           
                !CMK removed 2nd write
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat_all(n)%lsp,'buddep_dat_lsp')
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat_all(n)%cp ,'buddep_dat_cp')
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat_all(n)%cond ,'buddep_dat_nat')
                call io_write(io,im(n),'im',jm(n),'jm', ntrace,'ntrace', &
                     buddep_dat_all(n)%dry ,'buddep_dat_dry')
                call io_write(io,jm(n),'jm',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_x1 ,'budget_flux_x1')
                call io_write(io,im(n),'im',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_y1 ,'budget_flux_y1')
                call io_write(io,im(n),'im',jm(n),'jm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_z1 ,'budget_flux_z1')
                call io_write(io,jm(n),'jm',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_x2 ,'budget_flux_x2')
                call io_write(io,im(n),'im',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_y2 ,'budget_flux_y2')
                call io_write(io,im(n),'im',jm(n),'jm', ntracet,'ntracet', &
                     budget_flux_all(n)%flux_z2 ,'budget_flux_z2')
#else
                !CMK removed 2nd write sep2003
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat(n)%lsp,'buddep_dat_lsp')
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat(n)%cp ,'buddep_dat_cp')
                call io_write(io,im(n),'im',jm(n),'jm',nbud_vg,'nbud_v', &
                     ntrace,'ntrace',buddep_dat(n)%cond ,'buddep_dat_nat')
                call io_write(io,im(n),'im',jm(n),'jm', ntrace,'ntrace', &
                     buddep_dat(n)%dry ,'buddep_dat_dry')
                call io_write(io,jm(n),'jm',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux(n)%flux_x1 ,'budget_flux_x1')
                call io_write(io,im(n),'im',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux(n)%flux_y1 ,'budget_flux_y1')
                call io_write(io,im(n),'im',jm(n),'jm', ntracet,'ntracet', &
                     budget_flux(n)%flux_z1 ,'budget_flux_z1')
                call io_write(io,jm(n),'jm',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux(n)%flux_x2 ,'budget_flux_x2')
                call io_write(io,im(n),'im',lm(n),'lm', ntracet,'ntracet', &
                     budget_flux(n)%flux_y2 ,'budget_flux_y2')
                call io_write(io,im(n),'im',jm(n),'jm', ntracet,'ntracet', &
                     budget_flux(n)%flux_z2 ,'budget_flux_z2')
#endif
                !leave allocated for later use! deallocate( budg_dat(n)%nzong)
                deallocate( buddep_dat(n)%lsp)
                deallocate( buddep_dat(n)% cp)
                deallocate( buddep_dat(n)% cond)
                deallocate( buddep_dat(n)%dry)
                deallocate( buddep_dat_all(n)%lsp)
                deallocate( buddep_dat_all(n)% cp)
                deallocate( buddep_dat_all(n)% cond)
                deallocate( buddep_dat_all(n)%dry)

                deallocate( budget_flux(n)%flux_x1)
                deallocate( budget_flux(n)%flux_y1)
                deallocate( budget_flux(n)%flux_z1)
                deallocate( budget_flux_all(n)%flux_x1)
                deallocate( budget_flux_all(n)%flux_y1)
                deallocate( budget_flux_all(n)%flux_z1)
                deallocate( budget_flux(n)%flux_x2)
                deallocate( budget_flux(n)%flux_y2)
                deallocate( budget_flux(n)%flux_z2)
                deallocate( budget_flux_all(n)%flux_x2)
                deallocate( budget_flux_all(n)%flux_y2)
                deallocate( budget_flux_all(n)%flux_z2)

             end do

             print *, 'diagbudg: Close budget file region',region, sfend(io)

#ifdef MPI        
          end if ! root_t
#endif

       end if !region == nregions...


    case(3)
       !
       ! add wet and dry deposition fields to budget
       !
       if (region == 1) then
          buddryg=0.
          budwetg_lsp=0.
          budwetg_cp=0.
          budwetg_nat=0.
       endif
       do n=1,nt
          nglob=n+offsetn !WP! offset is zero on levels domain
          do nzone_v=1,nbud_vg
             do j=1,jm(region)
                do i=1,im(region)
                   nzone=budg_dat(region)%nzong(i,j)
                   budwetg_lsp(nzone,nzone_v,nglob) = &
                        budwetg_lsp(nzone,nzone_v,nglob) + &
                        buddep_dat(region)%lsp(i,j,nzone_v,nglob) 
                   budwetg_cp (nzone,nzone_v,nglob) = &
                        budwetg_cp (nzone,nzone_v,nglob) + &
                        buddep_dat(region)%cp (i,j,nzone_v,nglob) 
                    budwetg_nat (nzone,nzone_v,nglob) = &
                        budwetg_nat (nzone,nzone_v,nglob) + &
                        buddep_dat(region)%cond (i,j,nzone_v,nglob) 
                end do
             end do!j
          end do!nzone_v
       end do !nt
       if (region==nregions) then
          do n=1,ntracet
             ! compensate double counting wet deposition in convection
             budconvg(:,:,n) = budconvg(:,:,n) + budwetg_cp(:,:,n)
          end do
       endif
       ! dry deposition: processed parallel over levels --> only 
       !                 on PE processed the surface / other have zeros:
       do n=1,ntrace   !CMK changed....
          do j=1,jm(region)
             do i=1,im(region)
                nzone = budg_dat(region)%nzong(i,j)
                buddryg(nzone,1,n) = buddryg(nzone,1,n) + &
                     buddep_dat(region)%dry(i,j,n) 
             end do
          end do!j
       end do !

    case default
       call escape_tm('diagbudg: Wrong k entered the budget routines ' )
    end select

#ifdef MPI
  contains


    subroutine gather_budget
      implicit none
      integer :: nsend, region

      nsend=nbudg*nbud_vg*nmark
      call mpi_allreduce(budmarkg,budmarkg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbudg*nbud_vg*nreacw
      call mpi_allreduce(budrwg,budrwg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbudg*nbud_vg*nreac
      call mpi_allreduce(budrrg,budrrg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbudg*nbud_vg*njnum
      call mpi_allreduce(budrjg,budrjg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbudg*nbud_vg*ntrace
      call mpi_allreduce(budinig,budinig_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(buddryg,buddryg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwetg_lsp,budwetg_lsp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwetg_cp,budwetg_cp_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budwetg_nat,budwetg_nat_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budemig,budemig_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budstratg,budstratg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budconcg,budconcg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budchngg,budchngg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      nsend=nbudg*nbud_vg*ntracet
      call mpi_allreduce(budadvxg,budadvxg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budadvyg,budadvyg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budadvzg,budadvzg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      call mpi_allreduce(budconvg,budconvg_all,nsend,my_real, &
           mpi_sum,communicator,ierr)
      do region = 1, nregions
         nsend = im(region)*jm(region)*nbud_vg*ntrace
         call mpi_allreduce(buddep_dat(region)%lsp, &
              buddep_dat_all(region)%lsp, &
              nsend,my_real,mpi_sum,communicator,ierr)
         call mpi_allreduce(buddep_dat(region)%cp, &
              buddep_dat_all(region)%cp, &
              nsend,my_real,mpi_sum,communicator,ierr)
         nsend = im(region)*jm(region)*ntrace
         call mpi_allreduce(buddep_dat(region)%dry, &
              buddep_dat_all(region)%dry, &
              nsend,my_real,mpi_sum,communicator,ierr)
         nsend = jm(region)*lm(region)*ntracet
         call mpi_allreduce(budget_flux(region)%flux_x1, &
              budget_flux_all(region)%flux_x1, &
              nsend,my_real,mpi_sum,communicator,ierr)
         call mpi_allreduce(budget_flux(region)%flux_x2, &
              budget_flux_all(region)%flux_x2, &
              nsend,my_real,mpi_sum,communicator,ierr)
         nsend = im(region)*lm(region)*ntracet
         call mpi_allreduce(budget_flux(region)%flux_y1, &
              budget_flux_all(region)%flux_y1, &
              nsend,my_real,mpi_sum,communicator,ierr)
         call mpi_allreduce(budget_flux(region)%flux_y2, &
              budget_flux_all(region)%flux_y2, &
              nsend,my_real,mpi_sum,communicator,ierr)
         nsend = im(region)*jm(region)*ntracet
         call mpi_allreduce(budget_flux(region)%flux_z1, &
              budget_flux_all(region)%flux_z1, &
              nsend,my_real,mpi_sum,communicator,ierr)
         call mpi_allreduce(budget_flux(region)%flux_z2, &
              budget_flux_all(region)%flux_z2, &
              nsend,my_real,mpi_sum,communicator,ierr)
      end do

    end subroutine gather_budget
#endif

  end subroutine diagbudg



  subroutine budget_transportg(region,stat,proces,prev_step)
    !
    ! routine to evaluate the budget of advection
    ! routine is called before and after advection
    ! The difference is used to calculate the budget.
    ! Problems occur with zoome regions. Here advection is applied by
    ! writing x-adges and y-edges (in put_xedges, put_yedges).
    ! Knowledge of the previous step is required to
    ! calculate the budget....
    !
    use dims
    use global_data, only : region_dat,mass_dat
    use toolbox,     only : escape_tm
#ifdef MPI
    use mpi_const
#endif

    implicit none

    ! in/out
    integer,intent(in)   :: stat   ! stat = 0 before, stat = 1 after process
    integer,intent(in)   :: region
    character(len=*)     :: proces       ! the process (xyzvcs)
    character(len=*)     :: prev_step    ! the previous process.

    ! local
    real,dimension(:,:,:,:),pointer       :: rm
    integer,dimension(:,:) ,pointer       :: zoomed, edge
    integer :: i,j,l,n,nzone,nzone_v,lmr,nt,offsetn,nglob
    logical,dimension(:,:),allocatable    :: skip_flag

    ! start
    if (.not.apply_budget_global) return

#ifdef MPI
    if ( previous_par(region) /= 'tracer' )  &
         call escape_tm('budget_transportg: must be on tracer domain')
    if (ntracetloc==0) return
#endif

    rm => mass_dat(region)%rm_t 
    rm => mass_dat(region)%rm_t 
    lmr=lm(region)
#ifdef MPI
    nt=ntracetloc
    which_par='tracer'
    offsetn=sum(ntracet_ar(0:myid-1))
#else
    nt=ntracet
    offsetn=0
#endif

    zoomed => region_dat(region)%zoomed
    edge => region_dat(region)%edge

    if ( okdebug ) then
       print *, 'budget_transportg: region, stat, proces, prev_step = ',&
            region, stat, proces,' ', prev_step
    end if

    allocate(skip_flag(im(region),jm(region)))

    skip_flag = .false.

    where ( zoomed /= region .or. edge >= 1 )
       skip_flag = .true.
    end where

    if ( proces(1:4) == 'advx' .and. prev_step /= 'y' ) then
       ! include y-edges if XYZ sequence
       where(edge >= 2) 
          skip_flag = .false.    !only remaining problem: reduced grid!   
       end where
    else if ( proces(1:4) == 'advy' .and. prev_step /= 'x' ) then
       where ( edge == 1 .or. edge == 3 ) 
          skip_flag = .false.
       end where
    else if ( proces(1:4) == 'advz' .and. prev_step /= 'y' ) then
       ! zyx sequence: include interface
       where ( edge >= 1 ) 
          skip_flag = .false.
       end where
    else if ( proces(1:4) == 'conv' ) then   ! always interface
       where ( edge >= 1 ) 
          skip_flag = .false.
       end where
    end if

    if ( stat == 0 ) then ! save current tracer concentrations
       rmoldg = 0.0
       do n=1,nt
          nglob=n+offsetn  
          do l=1,lmr
             nzone_v = nzon_vg(l)
             do j=1,jm(region)
                do i=1,im(region)
                   if (skip_flag(i,j)) cycle
                   nzone=budg_dat(region)%nzong(i,j)
                   rmoldg(nzone,nzone_v,nglob) = &
                        rmoldg(nzone,nzone_v,nglob) + rm(i,j,l,n)
                end do
             end do
          end do
       end do
    else
       rmnewg = 0.0
       do n=1,nt  !WP! local value of ntracet
          nglob=n+offsetn  
          do l=1,lmr
             nzone_v = nzon_vg(l)
             do j=1,jm(region)
                do i=1,im(region)
                   if (skip_flag(i,j)) cycle
                   nzone=budg_dat(region)%nzong(i,j)
                   rmnewg(nzone,nzone_v,nglob) = &
                        rmnewg(nzone,nzone_v,nglob) + rm(i,j,l,n)
                end do
             end do
          end do
       end do

       select CASE(proces(1:4))
       case('advx')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvxg(:,:,nglob) = budadvxg(:,:,nglob) + &
                  (rmnewg(:,:,nglob)-rmoldg(:,:,nglob))/ra(nglob)*1e3
          end do
       case('advy')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvyg(:,:,nglob) = budadvyg(:,:,nglob) + &
                  (rmnewg(:,:,nglob)-rmoldg(:,:,nglob))/ra(nglob)*1e3
          end do
       case('advz')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budadvzg(:,:,nglob) = budadvzg(:,:,nglob) + &
                  (rmnewg(:,:,nglob)-rmoldg(:,:,nglob))/ra(nglob)*1e3
          end do
       case('conv')
          do n=1,nt  !WP! local value of ntracet
             nglob=n+offsetn  
             budconvg(:,:,nglob) = budconvg(:,:,nglob) + &
                  (rmnewg(:,:,nglob)-rmoldg(:,:,nglob))/ra(nglob)*1e3
          end do
       case default
          print*,'budget_transportg: no valid value for transport budget,', &
               ' budget not updated!'
       end select
    end if

    deallocate(skip_flag)

    nullify(rm)
    nullify(zoomed)
    nullify(edge)

  end subroutine budget_transportg


end module budget_global
