module emission_co2_ff
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform co2 ffsphere emissions 
  !
  !       interface
  !       ---------
  !       call emission_apply_ff
  !
  !       method
  !       ------
  !       subroutine is called from its general parent routine apply_emission
  !-----------------------------------------------------------------------
  use Dims,          only: nregions, nlon360,nlat180
  use emission_data, only: gridbox_per_month_to_flux
  implicit none

  private

  public   :: co2_fossil
  ! public routines:
  public   :: declare_emission_co2_ff, free_emission_co2_ff, calc_emission_co2_ff

  real,dimension(nlon360,nlat180),target :: co2_fossil


contains

  subroutine declare_emission_co2_ff

  implicit none
  integer                                      :: status

  ! nothing to be done

  end subroutine declare_emission_co2_ff

  subroutine calc_emission_co2_ff
  use dims,        only: nlon360, nlat180, im,jm, sec_month, newsrun, idate, newmonth, sec_year
  use io_save,     only: readtm3hdf
  use emission_data,	only: chardate
  use chem_param, only: xmc

  implicit none
  integer                                      :: status
  integer                                      :: region, imr, jmr
  integer, parameter                           :: annual_field = 0, rank2 = 2, level1 = 1
  character(len=10)                            :: cdate
  real                                         :: year2month

  if(newmonth) then
      call chardate(cdate,idate,4)
      call readtm3hdf('/p71/co2/peters/ff_input/co2ff.edgar_cdiac.seasnam.1x1.hdf'//trim(cdate),rank2,nlon360,nlat180,&
                         level1,idate(2)-1,co2_fossil,'edgarpattern_cdiactotal_seasnam')
      year2month = sec_month/sec_year
      co2_fossil=co2_fossil*year2month*1.e12 ! from PgC/gridbox/yr to kgC/gridbox/month
      call gridbox_per_month_to_flux(co2_fossil)   ! to kgC/m2/s
      co2_fossil=co2_fossil*1.e3/xmc  ! to mol/m2/s
  endif
   
  end subroutine calc_emission_co2_ff

   subroutine free_emission_co2_ff
    implicit none
    integer               :: region
   end subroutine free_emission_co2_ff

end module emission_co2_ff
