PROGRAM main
!
!    Kalman Filter solution to inverse problem.
!
!    TM5 is used as transport model
!
!    Changes to TM5 source code:
!
!    Wouter Peters, March 2005
!
!
  use GO              , only : TDate
  use GO              , only : GO_Print_Init, GO_Print_Done, gol, goPr
  use GO              , only : TrcFile, Init, Done, ReadRc
  use ParTools        , only : Par_Init, Par_Done
  use ParTools        , only : Par_Barrier
  use ParTools        , only : myid, root, npes
  use global_data     , only : rcfile
  use Meteo           , only : Meteo_Init, Meteo_Done
  use ModelIntegration, only : Proces_Init, Proces_Done
  use InitExit        , only : start, exitus
  use Toolbox         , only : escape_tm
  use enkf_tools   

  implicit none
  ! --- const ---------------------------------------
  
  character(len=*), parameter  ::  pname = 'Main'
   
  ! --- local ---------------------------------------

  integer           ::  status

  logical           ::  go_print_all
  logical           ::  go_print_apply
  logical           ::  go_print_trace
  logical           ::  go_print_prompt_pe
  
  ! --- begin ---------------------------------------

  !---------------!
  ! model startup !
  !---------------!
  
  ! ~~~ par
  
  ! init parallelisation
  call Par_Init( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! ~~~ messages
  
  ! read settings:
  call Init( rcF, rcfile, status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  call ReadRc( rcF, 'go.print.all', go_print_all, status, default=.false. )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  call ReadRc( rcF, 'go.print.prompt.pe', go_print_prompt_pe, status, default=npes>1 )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  call ReadRc( rcF, 'go.print.trace', go_print_trace, status, default=.false. )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  call Done( rcF, status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
    
  ! standard output by root only:
  go_print_apply = go_print_all .or. (myid==root)
  
  ! setup standard output processing:
  call GO_Print_Init( status, &
                        apply=go_print_apply, trace=go_print_trace, &
                        prompt_pe=go_print_prompt_pe, pe=myid )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  
  write (gol,'("")'); call goPr
  write (gol,'("*************************************************************")'); call goPr
  write (gol,'("***                                                       ***")'); call goPr
  write (gol,'("***                                                       ***")'); call goPr
  write (gol,'("***        Global Atmospheric Tracer Model TM5            ***")'); call goPr
  write (gol,'("***                                                       ***")'); call goPr
  write (gol,'("***                                                       ***")'); call goPr
  write (gol,'("*************************************************************")'); call goPr
  write (gol,'("")'); call goPr

  ! ~~~ meteo  

  ! setup meteo input:
  write (gol,'("setup meteo ...")'); call goPr
  call Meteo_Init( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! init processes
  write (gol,'("setup processes ...")'); call goPr
  call Proces_Init( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! ~~~ other
  
  ! set-up and read user input; 
  ! return time interval for which meteo was read:
  write (gol,'("start ...")'); call goPr
  call Start(.false., status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  call Par_Barrier

  call Filter_Init(status) ! initialize filter 
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  call Par_Barrier

  call CreateEnsemble  ! read/create prior mean and uncertainty

  call Par_Barrier

  if(.not.pseudo) call SampleState   ! run TM5 to create modeled CO2

  call Par_Barrier

  call Vectorize  ! gather results and create matrices for inversions

  call Par_Barrier

  call SaveState('prior',status)
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  call Par_Barrier

  if(.not.pseudo) call Invert     ! solve KF equations

  call Par_Barrier

  call SaveState('posterior',status)
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  call Par_Barrier

  call unVectorize  ! scatter results and deallocate matrices

  call Par_Barrier

  call AdvanceBackground

  call Par_Barrier

  call Vectorize(obsonly=.True.)  ! gather results (concentrations only)

  call Par_Barrier

  call SaveState('Final',status)
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  call Par_Barrier

  call unVectorize(obsonly=.True.)  ! free memory (concentrations only)

  call Par_Barrier
  !---------------!
  ! model finish  !
  !---------------!
  
  ! done processes
  call Proces_Done( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! close meteo files etc
  call Meteo_Done( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! done parallelisation
  call Par_Done( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)

  ! done standard output:
  call GO_Print_Done( status )
  if (status/=0) call escape_tm('Program Aborted in '//pname)
  
  ! store save file etc
  call exitus

END PROGRAM main
