!
! Module handles the surface data (needed for CBM4 chemistry)
!   o allocation of memory
!   o reading (on 1x1 resolution) (PE = root)
!   o coarsening to model grid (PE = root)
!   o distribute among PEs
!
module surface

#ifdef old_meteo
  
  use dims, only : nregions
  use chem_param, only : emis_data

  implicit none
  
  ! --- in/out -------------------------------------

  private

  public :: declare_surface_fields, read_surface_file, free_surface_fields
  public :: cp, lsp

  ! --- global variables ---------------------------
  
  type(emis_data),dimension(nregions),target  :: cp
  type(emis_data),dimension(nregions),target  :: lsp

  ! --- local ---------------------------

  ! nsurf_fields: surface fields to be coarsened
  integer,parameter                           :: nsurf_fields = 2
  character(len=10),dimension(nsurf_fields)   :: name_surffields = &
       (/ 'cp        ', 'lsp       ' /)


contains


  ! ===========================================================
  
  
  subroutine declare_surface_fields

    use Dims, only : im, jm
    
    ! --- local ----------------------------------

    integer  :: region,imr,jmr, n

    ! --- begin ----------------------------------

    do region = 1,nregions
       imr = im(region) ; jmr = jm(region)
       do n=1, nsurf_fields
          select case(name_surffields(n))
          case('cp')
             allocate(cp(region)%surf(imr,jmr))
          case('lsp')
             allocate(lsp(region)%surf(imr,jmr))
          case default
          end select
       end do
    end do

  end subroutine declare_surface_fields


  ! ***
  

  subroutine free_surface_fields
  
    use Dims, only : im, jm

    ! --- local ------------------------------------
    
    integer  :: region,imr,jmr, n

    ! --- begin ------------------------------------
    
    do region = 1,nregions
       imr = im(region) ; jmr = jm(region)
       do n=1, nsurf_fields
          select case(name_surffields(n))
          case('cp')
             deallocate(cp(region)%surf)
          case('lsp')
             deallocate(lsp(region)%surf)
          case default
          end select
       end do
    end do

  end subroutine free_surface_fields


  ! ***
  

  subroutine read_surface_file
    !
    use dims,      only : idate,revert,itau
    use Dims,      only : im, jm
    use Dims,      only : nlon360,nlat180
    use Dims,      only : okdebug
    use toolbox,   only : coarsen_emission, escape_tm
    use datetime,  only : tau2date
    use Meteo    , only : ReadSurfaceField
#ifdef MPI   
    use mpi_const, only : myid,root,mpi_comm_world,ierr,my_real
#endif

    ! --- local -----------------------------------
    
    integer,dimension(6)  :: idater
    integer,parameter     :: add_field=0,avg_field=1
    integer               :: n, region

    real,dimension(:,:),allocatable :: readfield
    
    integer               ::  status

    ! --- begin ------------------------------------

#ifdef MPI   
    if(myid == root) then 
#endif
       select case(revert)
       case(-1)
          call tau2date(itau-3600*3, idater)   !backward run: >>?
       case( 1)
          idater = idate
       case default
       end select

       allocate(readfield(nlon360,nlat180))

       do n=1,nsurf_fields

          call ReadSurfaceField( readfield, name_surffields(n), idater, 3, status )
          if (status/=0) call escape_tm('read_surface_file: error from ReadSurfaceField')

          select case(name_surffields(n))
          case('cp')
             call coarsen_emission('cp',nlon360,nlat180,readfield,cp,avg_field)
          case('lsp')
             call coarsen_emission('lsp',nlon360,nlat180,readfield,lsp,avg_field)
          case default
          end select

       end do

       deallocate(readfield)
#ifdef MPI   
    endif   !CMK all PEs from here 
#endif

#ifdef MPI   
    do n=1,nsurf_fields
       do region = 1, nregions
          select case(name_surffields(n))
          case('cp')
             call mpi_bcast(cp(region)%surf,im(region)*jm(region), &
                  my_real,root,mpi_comm_world,ierr)
          case('lsp')
             call mpi_bcast(lsp(region)%surf,im(region)*jm(region), &
                  my_real,root,mpi_comm_world,ierr)
          case default
          end select
       end do
    end do
#endif

  end subroutine read_surface_file

#endif

end module surface
